@@ -33,7 +33,7 @@ class BaseCondition:
         :return: The re-parameterized output of the network.
         :rtype: `torch.Tensor`
 
-        .. note:: 
+        .. note::
             This method is **abstract** for BaseCondition
         """
         raise ValueError(f"Abstract {self.__class__.__name__} cannot be parameterized")  # pragma: no cover
@@ -150,7 +150,7 @@ class NoCondition(BaseCondition):
     """
 
     def parameterize(self, output_tensor, *input_tensors):
-        f"""Performs no re-parameterization, or identity parameterization, in this case.
+        r"""Performs no re-parameterization, or identity parameterization, in this case.
 
         :param output_tensor: Output of the neural network.
         :type output_tensor: `torch.Tensor`
@@ -525,7 +525,6 @@ class IBVP1D(BaseCondition):
 
 
 class DoubleEndedBVP1D(BaseCondition):
-      
     r"""A boundary condition on a 1-D range where :math:`x\in[x_0, x_1]`.
     The conditions should have the following parts:
 
@@ -553,6 +552,7 @@ class DoubleEndedBVP1D(BaseCondition):
         Dirichlet conditions (by specifying only ``x_min_val`` and ``x_max_val``) and ``force`` is set to True in
         EnsembleCondition's constructor.
     """
+
     def __init__(
             self, x_min, x_max,
             x_min_val=None, x_min_prime=None,
@@ -585,6 +585,7 @@ class DoubleEndedBVP1D(BaseCondition):
             if self.ith_unit is not None:
                 out = out[:, self.ith_unit].view(-1, 1)
             return out
+
         ux = ANN(x)
         if self.x_min_val is not None and self.x_max_val is not None:
             return self.parameterize(ux, x)
@@ -604,7 +605,7 @@ class DoubleEndedBVP1D(BaseCondition):
             return self.parameterize(ux, x, ux0, x0, ux1, x1)
         else:
             raise NotImplementedError('Sorry, this boundary condition is not implemented.')
-    
+
     def parameterize(self, u, x, *additional_tensors):
         r"""Re-parameterizes outputs such that the boundary conditions are satisfied.
 
@@ -664,34 +665,34 @@ class DoubleEndedBVP1D(BaseCondition):
             return self._parameterize_nn(u, x, x_tilde, *additional_tensors)
         else:
             raise NotImplementedError('Sorry, this boundary condition is not implemented.')
-    
-    
+
     # When we have Dirichlet boundary conditions on both ends of the domain:
     def _parameterize_dd(self, ux, x, x_tilde):
-        Ax = self.x_min_val * (1-x_tilde) + self.x_max_val * (x_tilde)
+        Ax = self.x_min_val * (1 - x_tilde) + self.x_max_val * (x_tilde)
         return Ax + x_tilde * (1 - x_tilde) * ux
-    
+
     # When we have Dirichlet boundary condition on the left end of the domain
     # and Neumann boundary condition on the right end of the domain:
     def _parameterize_dn(self, ux, x, x_tilde, ux1, x1):
         Ax = (1 - x_tilde) * self.x_min_val + 0.5 * x_tilde ** 2 * self.x_max_prime * (self.x_max - self.x_min)
         return Ax + x_tilde * (ux - ux1 + self.x_min_val - diff(ux1, x1) * (self.x_max - self.x_min))
-        #Ax = self.x_min_val + (x - self.x_min) * self.x_max_prime
-        #return Ax + x_tilde * (ux - (self.x_max - self.x_min) * diff(ux1, x1) - ux1)
-    
+        # Ax = self.x_min_val + (x - self.x_min) * self.x_max_prime
+        # return Ax + x_tilde * (ux - (self.x_max - self.x_min) * diff(ux1, x1) - ux1)
+
     # When we have Neumann boundary condition on the left end of the domain
     # and Dirichlet boundary condition on the right end of the domain:
     def _parameterize_nd(self, ux, x, x_tilde, ux0, x0):
         Ax = x_tilde * self.x_max_val - 0.5 * (1 - x_tilde) ** 2 * self.x_min_prime * (self.x_max - self.x_min)
         return Ax + (1 - x_tilde) * (ux - ux0 + self.x_max_val + diff(ux0, x0) * (self.x_max - self.x_min))
-        #Ax = self.x_max_val + (x - self.x_max) * self.x_min_prime
-        #return Ax + (1 - x_tilde) * (ux + (self.x_max - self.x_min) * diff(ux0, x0) - ux0)
-    
+        # Ax = self.x_max_val + (x - self.x_max) * self.x_min_prime
+        # return Ax + (1 - x_tilde) * (ux + (self.x_max - self.x_min) * diff(ux0, x0) - ux0)
+
     # When we have Neumann boundary conditions on both ends of the domain:
     def _parameterize_nn(self, ux, x, x_tilde, ux0, x0, ux1, x1):
-        Ax = - 0.5 * (1 - x_tilde) ** 2 * (self.x_max - self.x_min) * self.x_min_prime + 0.5 * x_tilde ** 2 * (self.x_max - self.x_min) * self.x_max_prime
-        return Ax + 0.5 * x_tilde  ** 2 * (ux - ux1 - 0.5 * diff(ux1, x1)*(self.x_max - self.x_min)) + \
-            0.5 * (1 - x_tilde) ** 2 * (ux - ux0 + 0.5 * diff(ux0, x0)*(self.x_max - self.x_min))
+        Ax = - 0.5 * (1 - x_tilde) ** 2 * (self.x_max - self.x_min) * self.x_min_prime + 0.5 * x_tilde ** 2 * (
+                self.x_max - self.x_min) * self.x_max_prime
+        return Ax + 0.5 * x_tilde ** 2 * (ux - ux1 - 0.5 * diff(ux1, x1) * (self.x_max - self.x_min)) + \
+               0.5 * (1 - x_tilde) ** 2 * (ux - ux0 + 0.5 * diff(ux0, x0) * (self.x_max - self.x_min))
 
 
 # TODO: reduce duplication
