@@ -1,6 +1,7 @@
 import torch
 import warnings
 import torch.nn as nn
+import inspect
 from inspect import signature
 from abc import ABC, abstractmethod
 from itertools import chain
@@ -15,6 +16,10 @@ from neurodiffeq.generators import Generator2D
 from neurodiffeq.function_basis import RealSphericalHarmonics
 
 
+def _requires_closure(optimizer):
+    return inspect.signature(optimizer.step).parameters.get('closure').default == inspect._empty
+
+
 class BaseSolver(ABC):
     r"""A class for solving ODE/PDE systems.
 
@@ -257,17 +262,17 @@ class BaseSolver(ABC):
         r"""Generate the next validation batch, register in ``self._batch_examples`` and return."""
         return self._generate_batch('valid')
 
-    def _do_optimizer_step(self):
+    def _do_optimizer_step(self, closure=None):
         r"""Optimization procedures after gradients have been computed. Usually ``self.optimizer.step()`` is sufficient.
         At times, users can overwrite this method to perform gradient clipping, etc. Here is an example::
 
             import itertools
             class MySolver(Solver)
-                def _do_optimizer_step(self):
+                def _do_optimizer_step(self, closure=None):
                     nn.utils.clip_grad_norm_(itertools.chain([net.parameters() for net in self.nets]), 1.0, 'inf')
-                    self.optimizer.step()
+                    self.optimizer.step(closure=closure)
         """
-        self.optimizer.step()
+        self.optimizer.step(closure=closure)
 
     def _run_epoch(self, key):
         r"""Run an epoch on train/valid points, update history, and perform an optimization step if key=='train'.
@@ -278,47 +283,69 @@ class BaseSolver(ABC):
         .. note::
             The optimization step is only performed after all batches are run.
         """
+        if self.n_batches[key] <= 0:
+            # XXX maybe we should append NaN to metric history?
+            return
         self._phase = key
         epoch_loss = 0.0
+        batch_loss = 0.0
         metric_values = {name: 0.0 for name in self.metrics_fn}
 
+        # Zero the gradient only once, before running the batches. Gradients of different batches are accumulated.
+        if key == 'train' and not _requires_closure(self.optimizer):
+            self.optimizer.zero_grad()
+
         # perform forward pass for all batches: a single graph is created and release in every iteration
         # see https://discuss.pytorch.org/t/why-do-we-need-to-set-the-gradients-manually-to-zero-in-pytorch/4903/17
         for batch_id in range(self.n_batches[key]):
             batch = self._generate_batch(key)
-            funcs = [
-                self.compute_func_val(n, c, *batch) for n, c in zip(self.nets, self.conditions)
-            ]
 
-            for name in self.metrics_fn:
-                value = self.metrics_fn[name](*funcs, *batch).item()
-                metric_values[name] += value
-            residuals = self.diff_eqs(*funcs, *batch)
-            residuals = torch.cat(residuals, dim=1)
-            loss = self.criterion(residuals) + self.additional_loss(funcs, key)
+            def closure(zero_grad=True):
+                nonlocal batch_loss
+                if key == 'train' and zero_grad:
+                    self.optimizer.zero_grad()
+                funcs = [
+                    self.compute_func_val(n, c, *batch) for n, c in zip(self.nets, self.conditions)
+                ]
 
-            # normalize loss across batches
-            loss /= self.n_batches[key]
+                for name in self.metrics_fn:
+                    value = self.metrics_fn[name](*funcs, *batch).item()
+                    metric_values[name] += value
+                residuals = self.diff_eqs(*funcs, *batch)
+                residuals = torch.cat(residuals, dim=1)
+                loss = self.criterion(residuals) + self.additional_loss(funcs, key)
+
+                # accumulate gradients before the current graph is collected as garbage
+                if key == 'train':
+                    loss.backward()
+                    batch_loss = loss.item()
+                return loss
 
-            # accumulate gradients before the current graph is collected as garbage
             if key == 'train':
-                loss.backward()
-            epoch_loss += loss.item()
+                if _requires_closure(self.optimizer):
+                    # If `closure` is required by `optimizer.step()`, perform a step for every batch
+                    self._do_optimizer_step(closure=closure)
+                else:
+                    # Otherwise, only perform backward propagation.
+                    # Optimizer step will be performed only once outside the for-loop (i.e. after all batches).
+                    closure(zero_grad=False)
+                epoch_loss += batch_loss
+            else:
+                epoch_loss += closure().item()
 
         # calculate mean loss of all batches and register to history
-        self._update_history(epoch_loss, 'loss', key)
+        self._update_history(epoch_loss / self.n_batches[key], 'loss', key)
 
-        # perform optimization step when training
-        if key == 'train':
+        # perform the optimizer step after all batches are run (if optimizer.step doesn't require `closure`)
+        if key == 'train' and not _requires_closure(self.optimizer):
             self._do_optimizer_step()
-            self.optimizer.zero_grad()
-        # update lowest_loss and best_net when validating
-        else:
+        if key == 'valid':
             self._update_best()
 
         # calculate average metrics across batches and register to history
         for name in self.metrics_fn:
-            self._update_history(metric_values[name] / self.n_batches[key], name, key)
+            self._update_history(
+                metric_values[name] / self.n_batches[key], name, key)
 
     def run_train_epoch(self):
         r"""Run a training epoch, update history, and perform gradient descent."""
@@ -359,12 +386,9 @@ class BaseSolver(ABC):
 
         monitor = kwargs.pop('monitor', None)
         if monitor:
-            warnings.warn("Passing `monitor` is deprecated and ignored, "
+            warnings.warn("Passing `monitor` is deprecated, "
                           "use a MonitorCallback and pass a list of callbacks instead")
-            if not getattr(monitor, 'check_every', None):
-                raise AttributeError(f'{monitor} doesn\'t have a `check_every` attribute')
-            if monitor.check_every is None:
-                monitor.check_every = 100  # legacy default `check_every` for monitors
+            callbacks = [monitor.to_callback()] + list(callbacks)
         if kwargs:
             raise ValueError(f'Unknown keyword argument(s): {list(kwargs.keys())}')
 
@@ -381,14 +405,6 @@ class BaseSolver(ABC):
             for cb in callbacks:
                 cb(self)
 
-            if monitor:
-                if self.local_epoch % monitor.check_every == 0 or self.local_epoch == max_epochs:
-                    monitor.check(
-                        self.nets,
-                        self.conditions,
-                        history=self.metrics_history,
-                    )
-
     @abstractmethod
     def get_solution(self, copy=True, best=True):
         r"""Get a (callable) solution object. See this usage example:
