@@ -1,3 +1,4 @@
+import pytest
 import torch
 import torch.nn as nn
 from torch import sin, cos
@@ -11,8 +12,11 @@ from neurodiffeq.operators import spherical_div
 from neurodiffeq.operators import spherical_laplacian
 from neurodiffeq.operators import spherical_vector_laplacian
 
-torch.manual_seed(42)
-np.random.seed(42)
+
+@pytest.fixture(autouse=True)
+def magic():
+    torch.manual_seed(42)
+    np.random.seed(42)
 
 
 class HarmonicsNN(nn.Module):
@@ -28,26 +32,38 @@ class HarmonicsNN(nn.Module):
 
 
 EPS = 1e-4
-n_points = 1024
-r_min = 1.0
-r_max = 1.0
-generator = GeneratorSpherical(n_points, r_min=r_min, r_max=r_max)
-r, theta, phi = [t.reshape(-1, 1) for t in generator.get_examples()]
 
-degrees = list(range(10))
-harmoincs_fn = ZonalSphericalHarmonics(degrees=degrees)
 
-F_r, F_theta, F_phi = [HarmonicsNN(degrees, harmoincs_fn) for _ in range(3)]
-vector_u = (F_r(r, theta, phi), F_theta(r, theta, phi), F_phi(r, theta, phi))
+@pytest.fixture
+def x():
+    n_points, r_min, r_max = 1024, 1.0, 10.0
+    g = GeneratorSpherical(n_points, r_min=r_min, r_max=r_max)
+    return [t.reshape(-1, 1) for t in g.get_examples()]
 
-f = HarmonicsNN(degrees, harmoincs_fn)
-scalar_u = f(r, theta, phi)
 
-curl = lambda a, b, c: spherical_curl(a, b, c, r, theta, phi)
-grad = lambda a: spherical_grad(a, r, theta, phi)
-div = lambda a, b, c: spherical_div(a, b, c, r, theta, phi)
-lap = lambda a: spherical_laplacian(a, r, theta, phi)
-vec_lap = lambda a, b, c: spherical_vector_laplacian(a, b, c, r, theta, phi)
+@pytest.fixture
+def degrees():
+    return list(range(10))
+
+
+@pytest.fixture
+def harmonics_fn(degrees):
+    return ZonalSphericalHarmonics(degrees=degrees)
+
+
+@pytest.fixture
+def F(degrees, harmonics_fn):
+    return [HarmonicsNN(degrees, harmonics_fn) for _ in range(3)]
+
+
+@pytest.fixture
+def U(F, x):
+    return list(map(lambda f: f(*x), F))
+
+
+@pytest.fixture
+def u(degrees, harmonics_fn, x):
+    return HarmonicsNN(degrees, harmonics_fn)(*x)
 
 
 def is_zero(t):
@@ -62,38 +78,38 @@ def is_zero(t):
         raise ValueError(f"t must be list, tuple or tensor; got {type(t)}")
 
 
-def test_div_curl():
-    curl_u = curl(*vector_u)
-    div_curl_u = div(*curl_u)
+def test_div_curl(U, x):
+    curl_u = spherical_curl(*U, *x)
+    div_curl_u = spherical_div(*curl_u, *x)
     assert is_zero(div_curl_u), div_curl_u
 
 
-def test_curl_grad():
-    grad_u = grad(scalar_u)
-    curl_grad_u = curl(*grad_u)
+def test_curl_grad(u, x):
+    grad_u = spherical_grad(u, *x)
+    curl_grad_u = spherical_curl(*grad_u, *x)
     assert is_zero(curl_grad_u), curl_grad_u
 
 
-def test_div_grad():
-    grad_u = grad(scalar_u)
-    div_grad_u = div(*grad_u)
-    lap_u = lap(scalar_u)
+def test_div_grad(u, x):
+    grad_u = spherical_grad(u, *x)
+    div_grad_u = spherical_div(*grad_u, *x)
+    lap_u = spherical_laplacian(u, *x)
     delta = div_grad_u - lap_u
     assert is_zero(delta), delta
 
 
-def test_laplacian():
-    test_div_grad()
+def test_laplacian(u, x):
+    test_div_grad(u, x)
 
 
-def test_curl_curl():
-    curl_curl_u = curl(*curl(*vector_u))
-    grad_div_u = grad(div(*vector_u))
-    vec_lap_u = vec_lap(*vector_u)
+def test_curl_curl(U, x):
+    curl_curl_u = spherical_curl(*spherical_curl(*U, *x), *x)
+    grad_div_u = spherical_grad(spherical_div(*U, *x), *x)
+    vec_lap_u = spherical_vector_laplacian(*U, *x)
 
     vec_delta = [cc - (gd - vl) for cc, gd, vl in zip(curl_curl_u, grad_div_u, vec_lap_u)]
     assert is_zero(vec_delta), vec_delta
 
 
-def test_vec_laplacian():
-    test_curl_curl()
+def test_vec_laplacian(U, x):
+    test_curl_curl(U, x)
